@@ -1,15 +1,16 @@
-
-from django.conf.urls.defaults import *
+try:
+    from django.conf.urls import *
+except:
+    from django.conf.urls.defaults import *
 
 
 urlpatterns = patterns('',
     url(r'^ajax_lookup/(?P<channel>[-\w]+)$',
         'ajax_select.views.ajax_lookup',
-        name = 'ajax_lookup'
+        name='ajax_lookup'
     ),
     url(r'^add_popup/(?P<app_label>\w+)/(?P<model>\w+)$',
         'ajax_select.views.add_popup',
-        name = 'add_popup'
+        name='add_popup'
     )
 )
-
